### 2024年3月17日开发日志

查找资料对项目加深理解以及初步的设计思路：

#### Linux内存管理中的slab分配器
Linux内核中基于伙伴算法实现的分区页框分配器适合大块内存的请求，它所分配的内存区是以页框为基本单位的。对于内核中小块连续内存的请求，比 如说几个字节或者几百个字节，如果依然分配一个页框来来满足该请求，那么这很明显就是一种浪费，即产生内部碎片（internal fragmentation）。为了解决小块内存的分配，Linux内核基于Solaris 2.4中的slab分配算法实现了自己的slab分配器。除此之外，slab分配器另一个主要功能是作为一个高速缓存，它用来存储内核中那些经常分配并释放的对象。

#### slab分配器的基本原理
slab分配器中用到了对象这个概念，所谓对象就是内核中的数据结构以及对该数据结构进行创建和撤销的操作。它的基本思想是将内核中经常使用的对象 放到高速缓存中，并且由系统保持为初始的可利用状态。比如进程描述符，内核中会频繁对此数据进行申请和释放。当一个新进程创建时，内核会直接从slab分 配器的高速缓存中获取一个已经初始化了的对象；当进程结束时，该结构所占的页框并不被释放，而是重新返回slab分配器中。如果没有基于对象的slab分 配器，内核将花费更多的时间去分配、初始化以及释放一个对象。

slab分配器有以下三个基本目标：

1. 减少伙伴算法在分配小块连续内存时所产生的内部碎片；

2. 将频繁使用的对象缓存起来，减少分配、初始化和释放对象的时间开销。

3. 通过着色技术调整对象以更好的使用硬件高速缓存；

#### slab分配器的结构
slab分配器为每种对象分配一个高速缓存，这个缓存可以看做是同类型对象的一种储备。每个高速缓存所占的内存区又被划分多个slab，每个slab是由一个或多个连续的页框组成。每个页框中包含若干个对象，既有已经分配的对象，也包含空闲的对象。slab分配器的大致组成图如下：
![alt text](images/image.png)

#### 初步设计思路
Linux内核中基于伙伴算法实现的分区页框分配器适合大块内存的请求，它所分配的内存区是以页框为基本单位的。有时候，我们不需要一下子分配很大的内存，对于内核中小块连续内存的请求，例如分配一个 task_struct 结构，只需要分配小块的内存，去存储这个进程描述结构的对象。为了满足对这种小内存块的需要，Linux 系统采用了一种被称为 slab 分配器的技术，用于分配称为 slab 的一小块内存。它的基本原理是从内存管理模块申请一整块页，然后划分成多个小块的存储池，用复杂的队列来维护这些小块的状态（状态包括：被分配了 / 被放回池子 / 应该被回收）
1. 抽象对象：
首先需要定义一个抽象的对象，用于表示被分配的内存块。这个对象包含用于管理内存块的各种属性和方法，例如内存块的大小、状态（已分配或未分配）、分配和释放等操作。
2. 设计类结构：
基于抽象对象的定义，设计一个或多个类来表示内存块对象及其管理器。考虑使用类继承、组合等面向对象的概念来构建类结构，使其能够清晰地表示内存块的层次结构和管理关系。
3. 实现内存分配器：
基于Linux内核slab分配器的原理，设计并实现一个内存分配器类，用于管理内存块的分配和释放。参考Linux内核slab分配器的算法，尝试优化并将其转化为面向对象的设计和实现。
4. 性能优化：
在实现过程中，考虑如何优化内存分配器的性能，包括减少内存碎片、提高分配和释放的效率等方面。可以使用一些常见的优化技术，例如内存池、预分配等。
5. 错误处理：
同时还需要考虑到内存分配过程中可能出现的错误情况，例如内存耗尽、分配失败等，设计相应的错误处理机制来保证系统的稳定性和健壮性。
6. 测试与调试：
在实现过程中，进行充分的测试和调试，确保系统能够稳定运行，并且能够正确地处理各种情况下的内存分配和释放操作。
