### 2024年4月19日开发日志

基于C++的SLAB分配器设计与实现方案，我们小组完成了代码的初步开发，其中包括：

1. slab_allocator.h
   - 使用类封装SLAB分配器，满足赛题中对象化slab分配器的要求
   - 用于在固定大小的内存池中管理固定大小的内存块
   - 通过位图管理内存块的使用情况，使用链表来管理多个slab

***部分数据结构和方法如下***

声明Slab类模板
```C++
template<size_t slab_size, size_t memory_size> class Slab;
```

用于表示SLAB分配器的头部信息的结构体模板
```C++
template<size_t slab_size, size_t memory_size, size_t max_blocks = memory_size / slab_size> struct SlabHeader {
    Slab<slab_size, memory_size>* prev, * next;
    Bitmap<max_blocks> mem_map;
    size_t free_blocks;
    size_t next_fit_block;
}
```

对slab类模板的定义
```C++
template<size_t slab_size, size_t memory_size> class Slab {
...
};
```

在当前SLAB中分配一个内存块，并返回该内存块的地址
```C++
void* alloc_in_current_slab(size_t block_index);
```

在新的SLAB中分配一个内存块，并返回该内存块的地址
```C++
void* alloc_in_new_slab();
```

释放当前SLAB中的一个内存块
```C++
void free_from_current_slab(size_t block_index);
```


2. malloc.cpp
   - 基于之前定义的SLAB分配器，实现了一个自定义的内存分配器
   - 可以根据不同的内存需求，选择不同大小的SLAB分配器进行内存分配
  
***部分代码如下***

设置页面大小为4KB
```C++
const size_t PAGE_SIZE = 0x1000;
```

声明了七个不同大小的SLAB分配器对象，分别用于管理不同大小的内存块
```C++
static Slab<0x010, PAGE_SIZE> slab_0x10;
static Slab<0x020, PAGE_SIZE> slab_0x20;
static Slab<0x040, PAGE_SIZE> slab_0x40;
static Slab<0x060, PAGE_SIZE> slab_0x60;
static Slab<0x100, PAGE_SIZE> slab_0x100;
static Slab<0x200, PAGE_SIZE> slab_0x200;
static Slab<0x300, PAGE_SIZE> slab_0x300;
```

根据请求的大小选择合适大小的SLAB分配器进行内存分配
```C++
void* custom_malloc(size_t size) {
    if (size < 0x10) {
        return slab_0x10.alloc();
    }
    else if (size < 0x20) {
        return slab_0x10.alloc();
    }
    else if (size < 0x40) {
        return slab_0x40.alloc();
    }
    else if (size < 0x60) {
        return slab_0x60.alloc();
    }
    else if (size < 0x100) {
        return slab_0x100.alloc();
    }
    else if (size < 0x200) {
        return slab_0x200.alloc();
    }
    else if (size < 0x500) {
        return slab_0x300.alloc();
    }
    else {
        return nullptr;
    }
}
```


3. Bitmap.h
   - 实现了一个简单的位图类
   - 用于管理SLAB分配器中内存块的使用情况
   - 提供一系列操作

***部分代码如下***

检查位图中指定位置的位是否被设置
```C++
#define CHECK_BIT(value, bit) ((value >> bit) & 1)
```

置指定位置的位为已使用状态
```C++
void set_used(unsigned position);
```

查找未使用的位，从指定位置开始搜索
```C++
unsigned find_unused(unsigned search_start = 0);
```

检查指定位置的位是否未使用
```C++
bool check_unused(unsigned position);
```