#include "slab_allocator.h"

// 页面大小为4KB
const size_t PAGE_SIZE = 0x1000;
// 声明了七个不同大小的SLAB分配器对象，分别用于管理不同大小的内存块
static Slab<0x010, PAGE_SIZE> slab_0x10;
static Slab<0x020, PAGE_SIZE> slab_0x20;
static Slab<0x040, PAGE_SIZE> slab_0x40;
static Slab<0x060, PAGE_SIZE> slab_0x60;
static Slab<0x100, PAGE_SIZE> slab_0x100;
static Slab<0x200, PAGE_SIZE> slab_0x200;
static Slab<0x300, PAGE_SIZE> slab_0x300;

// 初始化所有的SLAB分配器对象
void init() {
    slab_0x10.init();
    slab_0x20.init();
    slab_0x40.init();
    slab_0x60.init();
    slab_0x100.init();
    slab_0x200.init();
    slab_0x300.init();
}

// 根据请求的大小选择合适大小的SLAB分配器进行内存分配
void* custom_malloc(size_t size) {
    if (size < 0x10) {
        return slab_0x10.alloc();
    }
    else if (size < 0x20) {
        return slab_0x10.alloc();
    }
    else if (size < 0x40) {
        return slab_0x40.alloc();
    }
    else if (size < 0x60) {
        return slab_0x60.alloc();
    }
    else if (size < 0x100) {
        return slab_0x100.alloc();
    }
    else if (size < 0x200) {
        return slab_0x200.alloc();
    }
    else if (size < 0x500) {
        return slab_0x300.alloc();
    }
    else {
        return nullptr;
    }
}

// 根据地址找到相应的SLAB分配器进行内存释放
void custom_free(void* address) {
    slab_0x10.free(address);
    slab_0x20.free(address);
    slab_0x40.free(address);
    slab_0x60.free(address);
    slab_0x100.free(address);
    slab_0x200.free(address);
    slab_0x300.free(address);
}